(* $Id: misc.ml,v 1.4 1999/01/11 04:43:47 garrigue Exp $ *)

#open"fvect";;
#open"fstring";;

let use_euc =
  let lang = try sys__getenv "LANG" with Not_found -> "" in
  (string_length lang >=2 && lang.[0] = `j` && lang.[1] = `a`)
;;

let print_wchar c =
  if c <= 0xff then print_char (char_of_int c) else begin
    print_char (char_of_int (c lsr 8));
    print_char (char_of_int (c land 0xff))
  end
;;

let string_of_array arr =
  let len = vect_length arr in
  if not use_euc then
    let s = create_string len in
    for i = 0 to len-1 do s.[i] <- char_of_int arr.(i) done;
    s
  else
  let len' = ref 0 in
  for i = 0 to len - 1 do
    incr len';
    if arr.(i) > 0xff then incr len'
  done;
  let s = create_string !len' and j = ref 0 in
  for i = 0 to len - 1 do
    let c = arr.(i) in
    if c <= 0xff then s.[!j] <- char_of_int c else begin
      s.[!j] <- char_of_int (c lsr 8);
      incr j;
      s.[!j] <- char_of_int (c land 0xff)
    end;
    incr j
  done;
  s
;;

let array_of_string s =
  let len = string_length s in
  if not use_euc then
    let arr = make_vect len 32 in
    for i = 0 to len-1 do arr.(i) <- int_of_char s.[i] done;
    arr
  else
  let len' = ref 0 and i = ref 0 in
  while !i < len do
    if s.[!i] >= `\160` then incr i;
    incr i; incr len'
  done;
  let arr = make_vect !len' 32 and i = ref 0 in
  for j = 0 to !len' - 1 do
    let c = int_of_char s.[!i] in
    if c < 160 or !i = len - 1 then arr.(j) <- c else begin
      incr i;
      arr.(j) <- c lsl 8 + int_of_char s.[!i]
    end;
    incr i
  done;
  arr
;;

let rec nth n l =
  if n = 0 then hd l else nth (n-1) (tl l)
;;

let rec flatten = function
    [] -> []
  | l :: ll -> l @ flatten ll
;;

let split_last l =
  match rev l with
    last :: rl -> rev rl, last
  | [] -> raise (Invalid_argument "misc__split_last")
;;

let rec do_list3 f l1 l2 l3 =
  match l1,l2,l3 with
    [],[],[] -> ()
  | a::l1, b::l2, c::l3 -> f a b c; do_list3 f l1 l2 l3
  | _ -> raise (Invalid_argument "do_list3")
;;

let rec add_bindings l map =
  it_list (fun map (key,data) -> map__add key data map) map l
;;
